#include "uart.h"

/*
        PIC UART
        Generic UART, Interrupt Driven

  Due to interfacing with an older Linux system, this
  command based protocol was written. It can easily be
  changed to a one-byte ASCII based system, which is popular.
  
  Being fixed-bytes, it interfaces very well with LabView, as you
  can easily tell Labview where in the string the data is.

  Commands are 3 bytes
  Each command has set & get value:
  SET = >    ,   GET = ?   1 byte

  Command format: ccc s/g (data)
  ccc - 3 byte command
  s/g - 1 byte
  data - ASCII data (variable length)

  Since we are defining the interface, we know
  what to expect & what position in the rxBuffer
  things will be in.
  
  rxBuf:
  0 to 2      command bytes
  3           SET/get byte
  4           start of data*
  
  *data must be the LAST part, as we can easily
  apply STR functions to convert if necessary

  For the code itself, it is always easier to let the compiler
  do all the assignments when possible.  
  
  There is a command routine that tests the CMD bytes for a match,
  and if match is found assigns it a enumerated value.  This way 
  we can easily add commands by entering it into the ENUM section.

  Hopefully this will become more clear as you look over the code.

  INTERRUPT:
  test each byte for your end-of-line character; the most common
  is simply a carriage return (ASCII 13), noted in code as <CR>

  NOTES:
  - all the NOP's make for nice break points when debugging

*/

#define RX_BUF_LEN 16            // 16 byte buffer
#define RS_GETSET       3        // get/set byte position
#define RS_DATA         4        // start of data bytes

/*
  enumerate commands to make life easy, let the compiler
  do all the assignments. 0 = BAD COMMAND!! So start at 1!
*/
enum cmds
{
    MOD=1,     // Model
    DAC,       // DAC
    ADC,       // ADC
    SET        // Set port bits
};
/*
  you can easily add commands now, and let the compiler
  handle assigning numbers to it. That way all we see
  is human readable text.
*/

volatile unsigned char rxBuf[RX_BUF_LEN];  // declare buffer
volatile char rxptr = 0;
volatile char rx_lastcmd;
volatile char rx_packet;
volatile char rx_ptr;
volatile char cmd;
int iTemp;
char cTemp;
volatile char cmpChar;
char getset = 0;
unsigned short cmd_addr;
unsigned long iLong;
float fTemp;
volatile char chan;

// soem fake vaules
char model[10]="MikroXYZ\r";
float adcvalue = 1.234;
int dacvalue = 0;

/*****************************************************/
/*************** function prototypes *****************/
/*****************************************************/
void clear_rx(void);
char check_rx(void);
void rs232_init(void);
void PrintHandler(char);
void rx_interrupt(void);


/*****************************************************/
/****************** functions ************************/
/*****************************************************/
char check_rx(void)
{
     char loop;
     char res = 0;
     res = 0;

     // search for CR (13) OR LF (10)
     for(loop=0;loop<RX_BUF_LEN;loop++)
     {
        if(rxBuf[loop]==13 || rxBuf[loop]==10)
        {
           res = 1;
           asm nop;
        }
     }
     return res;
}

void clear_rx(void)
{
     rx_packet = 0;
     rx_ptr = 0;
     memset(rxBuf,'\0',strlen(rxBuf));
     asm nop;
}


/*
  PrintHandler used in conjunctio with PRINTOUT in
  MicroC Pro library to send SPRINTF type strings
  to UART, one byte at a time. Can include all
  escapes, formatting, etc. in ANSI C SPRINTF sends
  data to screen, PRINTOUT sends wherever we want,
  in this case the UART.

  PrintOut(PrintHandler, "number &u4 is a 4 digit integer", 1234);
*/
void PrintHandler(char c)
{
     UART1_Write(c);
}

void rs232_init(void)
{
     clear_rx();
     rx_packet = 0;
     rx_ptr = 0;
     UART1_Init(9600);       // init UART
     asm nop;
}

/*
  command & channel address decoder...
  Yes, this could be implemented right in the
  decoder routine, but this keep the decoder routine
  MUCH cleaner...
*/
char my_cmd(void)
{
     char cm[3]; // temp buffer for cmd btyes

     // get command bytes
     cm[0]=toupper(rxBuf[0]);
     cm[1]=toupper(rxBuf[1]);
     cm[2]=toupper(rxBuf[2]);
     asm nop;
     /*
       now we have the three command bytes.
       we compare against strings
     */

     cmpChar = strncmp("MOD",&cm[0],3);
     asm nop;
     if(cmpChar==0)
     {
          asm nop;
          return MOD;
     }

     cmpChar = strncmp("DAC",&cm[0],3);
     asm nop;
     if(cmpChar==0)
     {
          asm nop;
          return DAC;
     }

     cmpChar = strncmp("ADC",&cm[0],3);
     asm nop;
     if(cmpChar==0)
     {
          asm nop;
          return ADC;
     }


     cmpChar = strncmp("SET",&cm[0],3);
     asm nop;
     if(cmpChar==0)
     {
          asm nop;
          return SET;
     }

     // if we make it here, invalid command or comms trashed!
     asm nop;
     return 0;    // invalid (thats why ENUMS start at 1!!)
}


/*
  <CR> detected in RXbuf, so we have a full command
*/
void decode_rs232(void)
{
      cmd =  my_cmd();   // get cmd & set channel
      asm nop;
      asm nop;
      getset = rxBuf[RS_GETSET];          // get or set
      asm nop;
      asm nop;
      /*
        Use a switch on 'cmd' to determine which command
      */
      switch(cmd)         // switch cmd
      {
            case MOD:     // model #
                  if(getset=='?') // get value
                  {
                      UART_Write_Text(&model);          // write out data
                      asm nop;
                  }
                  else if(getset=='>')       // set
                  {
                       // no SET allowed
                      asm nop;
                  }
                  break;

            case DAC:    // DAC value
                  if(getset=='?') // get
                  {
                      dacvalue;
                      PrintOut(PrintHandler, "%u\r",dacvalue);
                      asm nop;
                  }
                  else if(getset=='>')       // set
                  {
                      iTemp = atoi(&rxBuf[RS_DATA]);     // get value
                      dacvalue = iTemp;                  // assign
                      asm nop;
                  }
                  break;

            case ADC:    // get ADC value
                  if(getset=='?') // get
                  {
                      // put out on RS232
                      PrintOut(PrintHandler, "%5.0f\r",adcvalue);
                      asm nop;
                  }
                  else if(getset=='>')       // SET
                  {
                      // not allowed
                  }
                  asm nop;
                  break;

            case SET:    // SET PORTB
                  if(getset=='?') // get
                  {
                      cTemp = PORTB;    // grab PORTB
                      iTemp = cTemp;    // promote to int
                      PrintOut(PrintHandler, "%X\r",iTemp);
                      asm nop;
                  }
                  else if(getset=='>')       // SET
                  {
                      iTemp = atoi(&rxBuf[RS_DATA]);     // get value
                      cTemp = iTemp & 0xFF;
                      PORTB = cTemp;
                      asm nop;
                  }
                  asm nop;
                  break;


            default:    // bad command or comms trashed!
                  UART1_Write_Text("FAIL\r");
                  asm nop;
                  break;
      } // end switch
      clear_rx();    // always flush buffer
}