/*
  UHV_UART.C
  
  Commands are 4 bytes (set & get) to simplifly
  interfacing to external terminal programs

  Command format: AAcccc_(data)
  AA - Address 2 char 0 to 99
  cccc - 4 char command
  optional SPACE
  data - ASCII data
  
  GET return format: cccc_(data)
  command(SPACE)data
  e.g. DAC? returns "DAC?_1234"

  **All SET commands return 'ACK_' or 'NAK_'.

  EPIC Commands (> = set , ? = get):
  ** SET commads **
  DAC>     set dac value    0 to 4095
  HIV>     set hi voltage   6000 max
  ADR>     set address 0 to 99
  STR>     set soft trip
  
  ** GET commads **
  DAC?     get Dac value    0 to 4095
  HIV?     get hi voltage   6000 max
  COR?     get coarse current 1024 max
  ADC?     get ADC
  ADR?     get address 0 to 99
  STR?     get soft trip

  DEBUGGING COMMANDS:
  

*/

#include "xray_uart.h"
#include "radmon.h"

enum cmds 
{
    ADR=1,    // 0 = no valid cmd
    STL,
    DAC,
    HIV,
    ADC,
    COR,
    COP,
    DBA,
    DBB
};

volatile unsigned char rxbuf[32];
volatile char rxptr = 0;
volatile char cmd;
volatile int iTemp;
volatile char txtCmd[]="    ";
volatile char cmpChar;
char getset = 0;

sbit UART_EN at RA4_bit;
sbit UART_EN_Direction at TRISA4_bit;

/*****************************************************/
/*************** function prototypes *****************/
/*****************************************************/
void decode_rx(void);
void clear_rx(void);
void uhvuart_init(void);
char check_rx(void);


/*****************************************************/
/****************** functions ************************/
/*****************************************************/


/* slcd
   Function for sending STRINGS to SLCD via UART1
   ASSUMES UART1 INITIALIZED!
   USE: slcd("hello world")
   NOTE: can use escape type commands, this
   routine sends only ASCII, doesn't interpret,
   e.g. slcd("Hello \n world Number \"send in quotes \""
   sends Hello (newline) then "send in quotes" (with quotes)
*/
void slcd(char *txt)
{
  while(*txt)
  {
     if(*txt > 0)  // to filter nulls
       UART1_Write(*txt++);
       Delay_us(100);
  }
  UART1_Write(13);  // all comms end with carraige ret

  asm nop;
}

// Function for sending string to UART
void UartWriteText(char *txt)
{
  while(*txt)
  {
    if (*txt > 47)
    {
     UART1_Write(*txt++);
    }
  }
  UART1_Write(13);
  asm nop;
  asm nop;

}


/*
  PrintHandler used in conjunctio with PRINTOUT in
  MicroC Pro library to send SPRINTF type strings
  to UART, one byte at a time. Can include all
  escapes, formatting, etc. in ANSI C SPRINTF sends
  data to screen, PRINTOUT sends wherever we want,
  in this case the UART.

  PrintOut(PrintHandler, "number &u4 is a 4 digit integer", 1234);
*/
void PrintHandler(char c)
{

  UART1_Write(c);
  Delay_us(100);

}


void print_copyright(void)
{
PrintOut(PrintHandler, "\r\n"
                       "UHV ION PUMP SUPPLY\r\n"
                       "(c)Jefferson Lab, 2010.\r\n"
                       "Accelerator Operations\r\n"
                       "Hardware: John Hansknect\r\n"
                       "Firmware: Steve Covert\r\n"
                       "\r\n"
        );
}

void clear_rx(void)
{
     char loop;
     for(loop=0;loop<SER_BUFFER_SIZE;loop++)
     {
        rxbuf[loop]=0xFF;
     }
     rxptr = 0;       // reset pointer

}

void uart_init(void)
{
    UART1_Init(9600);         // init UART
    UART_EN_Direction = 0;  // output enable for UART
    UART_EN = 1;
    rxptr = 0;
    clear_rx();
    asm nop;
}


void uart_en(char onoff)
{
    if(onoff == 1)
        UART_EN = 1;
    else
        UART_EN = 0;
    asm nop;
}


char check_rx(void)
{
     char loop;
     char result = 0;
     result = 0;

     // search for CR (13) OR LF (10)
     for(loop=0;loop<SER_BUFFER_SIZE;loop++)
     {
        if(rxbuf[loop]==13 || rxbuf[loop]==10)
        {
           result = 1;
           asm nop;
        }
     }
     return result;
}


char my_addr(void)
{
   unsigned short temp1;

   temp1 = rxbuf[0]-0x30;           // 1st digit
   temp1 *= 10;
   temp1 += rxbuf[1]-0x30;          // 2nd digit
   asm nop;
//   cmd_addr = temp1*10+temp2;     // calc address
   if(temp1 == Unit.address)
            return 1;
   else
            return 0;

}

//enum cmds {ADR,STR,DAC,HIV,ADC,COR};
char my_cmd(void)
{
     char cm[4];

     cm[0]=toupper(rxBuf[2]);
     cm[1]=toupper(rxBuf[3]);
     cm[2]=toupper(rxBuf[4]);
     cm[3]=toupper(rxBuf[5]);

     asm nop;

     cmpChar = strncmp("COR",&cm[0],3);
     asm nop;
     if(cmpChar==0)return COR;

     cmpChar = strncmp("ADC",&cm[0],3);
     asm nop;
     if(cmpChar==0)return ADC;

     cmpChar = strncmp("DAC",&cm[0],3);
     asm nop;
     if(cmpChar==0)return DAC;

     cmpChar = strncmp("STL",&cm[0],3);
     asm nop;
     if(cmpChar==0)return STL;

     cmpChar = strncmp("ADR",&cm[0],3);
     asm nop;
     if(cmpChar==0)return ADR;

     cmpChar = strncmp("HIV",&cm[0],3);
     asm nop;
     if(cmpChar==0)return HIV;

     cmpChar = strncmp("DBA",&cm[0],3);
     asm nop;
     if(cmpChar==0)return DBA;

     cmpChar = strncmp("DBB",&cm[0],3);
     asm nop;
     if(cmpChar==0)return DBB;

     cmpChar = strncmp("COP",&cm[0],3);
     asm nop;
     if(cmpChar==0)return COP;

    asm nop;
    return 0;             // invalid
    
}


/*
  CR detected in RXbuf, so we have a full command
*/
void decode_rx(void)
{
    volatile unsigned short cmd_addr;
    volatile float volts;

    asm nop;

       if(my_addr())        // is it My address?
       {
         cmd =  my_cmd();   // get cmd
         getset = rxBuf[5];
         asm nop;
         switch(cmd)         // switch cmd
         {
           case DAC:    // set high voltage via DAC
                 if(getset=='>') // set
                 {
                     asm nop;
                     iTemp = atoi(&rxbuf[6]);   // 0 - 255 range input
                     if(iTemp <= 4095)           // verify in range
                     {
                       DAC_Write(iTemp);        // wrtie to 12bit DAC
                       Unit.dac_set=iTemp;
                       EEPROM_PUT((char*)&Unit,sizeof(Unit),0);
                       asm nop;
                     }
                 }
                 else if(getset=='?') // get
                 {
                     PrintOut(PrintHandler, "%3u\r\n",My.hivolt);
                     asm nop;
                 }
                 asm nop;
                 break;

            case HIV:    // set high voltage via VOLTS
                 if(getset=='>') // set
                 {
                     iTemp = atoi(&rxbuf[6]);   // 0 - 255 range input
                     if(iTemp <= 6000)  // verify in range 6KV max
                     {
                       // convert volts to DAC
                       volts = ((float)(iTemp))*.5464+633.0;
                       asm nop;
                       iTemp = (unsigned)(volts);
                       asm nop;
                       DAC_Write(iTemp);        // wrtie to 12bit DAC
                       Unit.dac_set=iTemp;
                       EEPROM_PUT((char*)&Unit,sizeof(Unit),0);
                       asm nop;
                     }
                     else
                     {
                        PrintOut(PrintHandler, "%3.0u\r\n",My.KV);
                     }
                 }
                 asm nop;
                 break;

            case ADR:    // set address
                 // SET ONLY
                 if(getset=='>') // set
                 {
                       iTemp = atoi(&rxbuf[6]);   // 0 - 99 range input
                       if(iTemp <= 99)            // verify in range
                       {
                         Unit.address=iTemp;
                       }
                       EEPROM_PUT((char*)&Unit,sizeof(Unit),0);
                 }
                 asm nop;
                 break;

            case '^':    // set broadcast true
                 Unit.broadcast = 1;
                 asm nop;
                 break;

            case ADC:    // get adc
                 if(getset=='?') // get
                 {
                      PrintOut(PrintHandler, "%8.0u\r\n",My.adc);
                 }
                 asm nop;
                 break;

            case DBA:    // get current amps
                 // GET OR SET OK
                 PrintOut(PrintHandler, "****************\r\n",My.amps);
                 PrintOut(PrintHandler, "AMPS: %8.2e\r\n",My.amps);
                 PrintOut(PrintHandler, "ADC: %8.0u\r\n",My.adc);
                 PrintOut(PrintHandler, "GAIN: %3.0u\r\n",(unsigned int)My.range);
                 PrintOut(PrintHandler, "KV: %3.0u\r\n",My.KV);
                 if(My_KV_OK)
                          PrintOut(PrintHandler, "HV ON\r\n");
                 else
                          PrintOut(PrintHandler, "HV OFF\r\n");
                 PrintOut(PrintHandler, "****************\r\n",My.amps);
                 PrintOut(PrintHandler, "\r\n");
                 PrintOut(PrintHandler, "\r\n");
                  asm nop;
                 break;

        /*
                 case 'D':    // set broadcast true
                 Unit.broadcast = 1;
                 asm nop;
                 break;
        */

                 case COR:    // get coarse current
                 if(getset=='?') // get
                 {
                       IntToStr(Unit.coarse,txt1);
                       Uart1_Write_Text(txt1);
                       Uart1_Write(0x0D);                    // CR
                 }
                 asm nop;
                 break;


                 case COP:    // print copyright
                 if(getset=='?') // get
                 {
                      print_copyright();
                 }
                 asm nop;
                 break;

                 case STL:    // set soft trip limit
                 if(getset=='>') // set
                 {
                       volts = atof(&rxbuf[6]);   // float input
                       if(volts <= 0.000095) // 95uA max
                       {
                           // store new value
                           Unit.soft_trip = volts;
                           EEPROM_PUT((char*)&Unit,sizeof(Unit),0);
                           PrintOut(PrintHandler, "SOFT TRIP: %8.2e\r\n",Unit.soft_trip);
                           asm nop;
                       }
                       else
                       {
                           PrintOut(PrintHandler, "SOFT TRIP: INVALID!\r\n",Unit.soft_trip);
                           asm nop;
                       }
                 }
                 else if(getset=='?') // get
                 {
                       PrintOut(PrintHandler, "%8.2e\r\n",Unit.soft_trip);
                       asm nop;
                 }
                 asm nop;
                 break;

            default:
                 Uart1_Write(0x0D);                    // CR
                 Uart1_Write(0x0A);                    // CR
                 Uart1_Write_Text("NO SUCH COMMAND");
                 Uart1_Write(0x0D);                    // CR
                 Uart1_Write(0x0A);                    // CR
                 asm nop;
                 break;
         } // end select
      } // end if cmd_addr = My_address
      else // got a CR or LF and no real data
      {
         clear_rx();
      }

}